@@ -37,6 +37,12 @@
 
 int   verbosity;
 
+char *safe_strdup(char *u) {
+  if(u)
+    return strdup(u);
+  return NULL;
+}
+
 void safe_strcpy(char *u, char * v, int usize) {
 /* Copies as much of v into u as it can assuming u is of size usize */
 /* guaranteed to terminate u with a '\0'.                           */
@@ -136,7 +142,7 @@ void init_randtable(void) {
 }
 
 /* Build a copy of the given expression (don't copy strings, of course) */
-E_list * copy_E_list(E_list * l);
+static E_list * copy_E_list(E_list * l);
 Exp * copy_Exp(Exp * e) {
     Exp * n;
     if (e == NULL) return NULL;
@@ -148,7 +154,7 @@ Exp * copy_Exp(Exp * e) {
     return n;
 }
 
-E_list * copy_E_list(E_list * l) {
+static E_list * copy_E_list(E_list * l) {
     E_list * nl;
     if (l == NULL) return NULL;
     nl = (E_list *) xalloc(sizeof(E_list));
@@ -407,10 +413,10 @@ path_get_dirname (const char	   *file_name)
 	  drive_colon_dot[2] = '.';
 	  drive_colon_dot[3] = '\0';
 
-	  return strdup (drive_colon_dot);
+	  return safe_strdup (drive_colon_dot);
 	}
 #endif
-    return strdup (".");
+    return safe_strdup (".");
     }
 
   while (base > file_name && IS_DIR_SEPARATOR (*base))
@@ -474,7 +480,7 @@ static char * get_datadir(void)
   char * data_dir = NULL;
 
 #ifdef ENABLE_BINRELOC
-  data_dir = strdup (BR_DATADIR("/link-grammar"));
+  data_dir = safe_strdup (BR_DATADIR("/link-grammar"));
 #elif defined(_WIN32)
   /* Dynamically locate library and return containing directory */
   HINSTANCE hInstance = GetModuleHandle(NULL);
@@ -559,7 +565,7 @@ static int random_state[2] = {0,0};
 static int random_count = 0;
 static int random_inited = FALSE;
 
-int step_generator(int d) {
+static int step_generator(int d) {
     /* no overflow should occur, so this is machine independent */
     random_state[0] = ((random_state[0] * 3) + d + 104729) % 179424673;
     random_state[1] = ((random_state[1] * 7) + d + 48611) % 86028121;
@@ -600,7 +606,7 @@ typedef struct {
 } Connector_set;
 #endif
 
-int connector_set_hash(Connector_set *conset, char * s, int d) {
+static int connector_set_hash(Connector_set *conset, char * s, int d) {
 /* This hash function only looks at the leading upper case letters of
    the string, and the direction, '+' or '-'.
 */
@@ -609,7 +615,7 @@ int connector_set_hash(Connector_set *conset, char * s, int d) {
     return (i & (conset->table_size-1));
 }
 
-void build_connector_set_from_expression(Connector_set * conset, Exp * e) {
+static void build_connector_set_from_expression(Connector_set * conset, Exp * e) {
     E_list * l;
     Connector * c;
     int h;
@@ -684,7 +690,7 @@ void free_listed_dictionary(Dict_node *dn) {
     }
 }
 
-int easy_match(char * s, char * t) {
+static int easy_match(char * s, char * t) {
 
     /* This is like the basic "match" function in count.c - the basic connector-matching
        function used in parsing - except it ignores "priority" (used to handle fat links) */
@@ -1296,7 +1302,7 @@ win32_getlocale (void)
   if (((ev = getenv ("LC_ALL")) != NULL && ev[0] != '\0')
       || ((ev = getenv ("LC_MESSAGES")) != NULL && ev[0] != '\0')
       || ((ev = getenv ("LANG")) != NULL && ev[0] != '\0'))
-    return strdup (ev);
+    return safe_strdup (ev);
 
   /* Use native Win32 API locale ID.  */
   lcid = GetThreadLocale ();
@@ -1702,7 +1708,7 @@ win32_getlocale (void)
       strcat (bfr, sl);
     }
 
-  return strdup (bfr);
+  return safe_strdup (bfr);
 }
 #endif
 
@@ -1734,19 +1740,19 @@ char * get_default_locale(void)
 #endif
 
   if(!locale)
-    locale = strdup (getenv ("LANG"));
+    locale = safe_strdup (getenv ("LANG"));
 
 #if defined(HAVE_LC_MESSAGES)
   if(!locale)
-    locale = strdup (setlocale (LC_MESSAGES, NULL));
+    locale = safe_strdup (setlocale (LC_MESSAGES, NULL));
 #endif
 
   if(!locale)
-    locale = strdup (setlocale (LC_ALL, NULL));
+    locale = safe_strdup (setlocale (LC_ALL, NULL));
 
   if(!locale || strcmp(locale, "C") == 0) {
     free(locale);
-    locale = strdup("en");
+    locale = safe_strdup("en");
   }
 
   /* strip off "@euro" from en_GB@euro */
